function sfun_poisson(block)
% implements a Poisson process
% outputs a rectangular pulse with rising edge at the time of an event
%
% parameter
%    lambda    intensity
%
  setup(block); 
end

%% -------------------------------------------------------------------
function setup(block)
  block.NumInputPorts  = 0;
  block.NumOutputPorts = 1;
  block.NumDialogPrms  = 1;

  block.SampleTimes = [-2 0]; 
  block.OutputPort(1).SamplingMode = 'sample';

  block.RegBlockMethod('PostPropagationSetup', @DoPostPropSetup);
  block.RegBlockMethod('InitializeConditions', @InitializeConditions);
  block.RegBlockMethod('Update', @Update);
  block.RegBlockMethod('Outputs', @Outputs);
end

%% -------------------------------------------------------------------
function DoPostPropSetup(block)
  block.NumDworks = 3;
  
  block.Dwork(1).Name            = 'tLast'; 
  block.Dwork(1).Dimensions      = 1;
  block.Dwork(1).DatatypeID      = 0;
  block.Dwork(1).Complexity      = 'Real';
  block.Dwork(1).UsedAsDiscState = true;

  block.Dwork(2).Name            = 'tInc'; 
  block.Dwork(2).Dimensions      = 1;
  block.Dwork(2).DatatypeID      = 0;
  block.Dwork(2).Complexity      = 'Real';
  block.Dwork(2).UsedAsDiscState = true;

  block.Dwork(3).Name            = 'state'; 
  block.Dwork(3).Dimensions      = 1;
  block.Dwork(3).DatatypeID      = 0;
  block.Dwork(3).Complexity      = 'Real';
  block.Dwork(3).UsedAsDiscState = true;
end
    
%% -------------------------------------------------------------------
function InitializeConditions(block)
  lambda = block.DialogPrm(1).Data;
  block.Dwork(1).Data = 0.0;
  block.Dwork(2).Data = -log(rand())/lambda;
  block.Dwork(3).Data = -1;
end

%% -------------------------------------------------------------------
function Update(block)
  lambda = block.DialogPrm(1).Data;
  tLast = block.Dwork(1).Data;
  tInc  = block.Dwork(2).Data;
  state = block.Dwork(3).Data;
  
  block.Dwork(1).Data = tLast + 0.5*tInc;
  block.Dwork(3).Data = -state;
    
  if (state == 1)     % compute next hit
     block.Dwork(2).Data = - log(rand())/lambda;
  end
end

%% -------------------------------------------------------------------
function Outputs(block)
  tLast = block.Dwork(1).Data;
  tInc  = block.Dwork(2).Data;
  state = block.Dwork(3).Data; 
  
  block.OutputPort(1).Data = state; 
  block.NextTimeHit = tLast + 0.5*tInc;
end
